@@ -30,11 +30,9 @@
 use dirs_next::home_dir;
 use eyre::{eyre, Result, WrapErr as _};
 use sn_launch_tool::Launch;
-use std::{
-    io,
-    path::PathBuf,
-    process::{Command, Stdio},
-};
+#[cfg(not(target_os = "windows"))]
+use std::process::{Command, Stdio};
+use std::{io, path::PathBuf};
 use structopt::StructOpt;
 use tokio::fs::{create_dir_all, remove_dir_all};
 use tokio::time::{sleep, Duration};
@@ -49,8 +47,8 @@ const SAFE_NODE_EXECUTABLE: &str = "sn_node.exe";
 
 const BASE_TRACING_DIRECTIVES: &str = "testnet=info,sn_launch_tool=debug";
 const NODES_DIR: &str = "local-test-network";
-const INTERVAL: Duration = Duration::from_millis(100);
-const DEFAULT_NODE_COUNT: u32 = 33;
+const DEFAULT_INTERVAL: &str = "10000";
+const DEFAULT_NODE_COUNT: u32 = 45;
 
 #[derive(Debug, StructOpt)]
 #[structopt(name = "testnet")]
@@ -58,6 +56,14 @@ struct Cmd {
     /// All nodes will be joining existing testnet, none will be started as a genesis node.
     #[structopt(long = "add")]
     add_nodes_to_existing_network: bool,
+
+    /// Interval in milliseconds between launching each of the nodes.
+    #[structopt(long = "interval", default_value = DEFAULT_INTERVAL)]
+    interval: u64,
+
+    /// Format logs as JSON.
+    #[structopt(long)]
+    json_logs: bool,
 }
 
 #[tokio::main]
@@ -77,30 +83,41 @@ async fn main() -> Result<()> {
         .await
         .wrap_err("Cannot create nodes directory")?;
 
-    let mut args = vec!["build", "--release"];
+    // TODO:: Remove this conditional compilation once the issue on Windows
+    // got resoved within the new version of Rust.
+    #[cfg(not(target_os = "windows"))]
+    // For Windows guys, rember to use
+    // `cargo build --release --features=always-joinable,test-utils --bins`
+    // before executing the testnet.exe.
+    {
+        let mut args = vec!["build", "--release"];
 
-    // Keep features consistent to avoid recompiling when possible
-    if cfg!(feature = "always-joinable") {
-        args.push("--features");
-        args.push("always-joinable");
+        // Keep features consistent to avoid recompiling when possible
+        if cfg!(feature = "always-joinable") {
+            args.push("--features");
+            args.push("always-joinable");
+        }
+        if cfg!(feature = "test-utils") {
+            args.push("--features");
+            args.push("test-utils");
+        }
+
+        info!("Building current sn_node");
+        debug!("Building current sn_node with args: {:?}", args);
+        Command::new("cargo")
+            .args(args.clone())
+            .current_dir("sn")
+            // .env("RUST_LOG", "debug")
+            // .env("RUST_BACKTRACE", "1")
+            .stdout(Stdio::inherit())
+            .stderr(Stdio::inherit())
+            .output()
+            .map_err(Into::into)
+            .and_then(|result| result.status.success().then(|| ()).ok_or_else(|| eyre!("Command exited with error")))
+            .wrap_err_with(|| format!("Failed to run build command with args: {:?}", args))?;
+
+        info!("sn_node built successfully");
     }
-    if cfg!(feature = "test-utils") {
-        args.push("--features");
-        args.push("test-utils");
-    }
-
-    info!("Building current sn_node");
-    debug!("Building current sn_node with args: {:?}", args);
-    let _child = Command::new("cargo")
-        .args(args.clone())
-        // .env("RUST_LOG", "debug")
-        // .env("RUST_BACKTRACE", "1")
-        .stdout(Stdio::inherit())
-        .stderr(Stdio::inherit())
-        .output()
-        .wrap_err_with(|| format!("Failed to run build command with args: {:?}", args))?;
-
-    info!("sn_node built successfully");
 
     run_network().await?;
 
@@ -142,7 +159,8 @@ pub async fn run_network() -> Result<()> {
     let node_count_str = node_count.to_string();
 
     // Let's create an args array to pass to the network launcher tool
-    let interval_str = INTERVAL.as_millis().to_string();
+    let interval_str = args.interval.to_string();
+
     let mut sn_launch_tool_args = vec![
         "sn_launch_tool",
         "--node-path",
@@ -160,6 +178,10 @@ pub async fn run_network() -> Result<()> {
         sn_launch_tool_args.push("--add")
     }
 
+    if args.json_logs {
+        sn_launch_tool_args.push("--json-logs");
+    }
+
     // If RUST_LOG was set we pass it down to the launch tool
     // so it's set for each of the nodes logs as well.
     let rust_log =
@@ -179,7 +201,7 @@ pub async fn run_network() -> Result<()> {
     Launch::from_iter_safe(&sn_launch_tool_args)?.run()?;
 
     // leave a longer interval with more nodes to allow for splits if using split amounts
-    let interval_duration = INTERVAL * node_count;
+    let interval_duration = Duration::from_millis(args.interval);
 
     sleep(interval_duration).await;
 
