@@ -18,8 +18,8 @@
 mod stats;
 
 use self::stats::NetworkStats;
-use crate::messaging::{system::SectionAuth, SectionAuthorityProvider};
-use crate::routing::{Error, Result, SectionAuthUtils, SectionAuthorityProviderUtils};
+use crate::messaging::system::SectionAuth;
+use crate::routing::{Error, Result, SectionAuthUtils, SectionAuthorityProvider};
 use bls::PublicKey as BlsPublicKey;
 use dashmap::{self, mapref::multiple::RefMulti, DashMap};
 use secured_linked_list::SecuredLinkedList;
@@ -29,7 +29,7 @@ use std::sync::Arc;
 use xor_name::{Prefix, XorName};
 
 /// Container for storing information about other sections in the network.
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, Serialize)]
 pub(crate) struct NetworkPrefixMap {
     /// Map of sections prefixes to their latest signed section authority providers.
     sections: Arc<DashMap<Prefix, SectionAuth<SectionAuthorityProvider>>>,
@@ -58,38 +58,53 @@ impl NetworkPrefixMap {
     /// the map.
     /// Returns a boolean indicating whether anything changed.
     //
-    // TODO: remove this form public API since we shall not allow any insert/update withot a
-    // proof chain, users shall have to call either `update` or `verify_with_chain_and_update` API.
-    pub(crate) fn insert(&self, sap: SectionAuth<SectionAuthorityProvider>) -> bool {
-        let prefix = sap.value.prefix();
+    // This is not a public API since we shall not allow any insert/update without a
+    // proof chain, users shall call either `update` or `verify_with_chain_and_update` API.
+    fn insert(&self, sap: SectionAuth<SectionAuthorityProvider>) -> bool {
+        let prefix = sap.prefix();
         // Don't insert if any descendant is already present in the map.
         if self.descendants(&prefix).next().is_some() {
             return false;
         }
 
-        let _ = self.sections.insert(prefix, sap);
+        let _prev = self.sections.insert(prefix, sap);
 
         let parent_prefix = prefix.popped();
         self.prune(parent_prefix);
         true
     }
 
-    /// Returns the known section that is closest to the given name, regardless of whether `name`
-    /// belongs in that section or not.
-    fn closest(&self, name: &XorName) -> Option<SectionAuth<SectionAuthorityProvider>> {
+    /// Returns the known section that is closest to the given name,
+    /// regardless of whether `name` belongs in that section or not.
+    /// If provided, it excludes any section matching the passed prefix.
+    fn closest(
+        &self,
+        name: &XorName,
+        exclude: Option<&Prefix>,
+    ) -> Option<SectionAuth<SectionAuthorityProvider>> {
         self.sections
             .iter()
+            .filter(|e| {
+                if let Some(prefix) = exclude {
+                    e.key() != prefix
+                } else {
+                    true
+                }
+            })
             .min_by(|lhs, rhs| lhs.key().cmp_distance(rhs.key(), name))
             .map(|e| e.value().clone())
     }
 
-    /// Returns the known section that is closest to the given name, regardless of whether `name`
-    /// belongs in that section or not. If there are no close matches, return a SAP from an opposite prefix.
+    /// Returns the known section that is closest to the given name,
+    /// regardless of whether `name` belongs in that section or not.
+    /// If there are no close matches in remote sections, return a SAP from an opposite prefix.
+    /// If provided, it excludes any section matching the passed prefix.
     pub(crate) fn closest_or_opposite(
         &self,
         name: &XorName,
+        exclude: Option<&Prefix>,
     ) -> Option<SectionAuth<SectionAuthorityProvider>> {
-        self.closest(name).or_else(|| {
+        self.closest(name, exclude).or_else(|| {
             self.sections
                 .iter()
                 .filter(|e| e.key().matches(&name.with_bit(0, !name.bit(0))))
@@ -126,45 +141,55 @@ impl NetworkPrefixMap {
     /// currently known SAP we are aware of for the Prefix.
     pub(crate) fn update(
         &self,
-        signed_section_auth: SectionAuth<SectionAuthorityProvider>,
+        signed_sap: SectionAuth<SectionAuthorityProvider>,
         proof_chain: &SecuredLinkedList,
     ) -> Result<bool> {
-        trace!("Attempting to update prefixmap");
-        let section_key = match self.section_by_prefix(&signed_section_auth.value.prefix()) {
+        let prefix = signed_sap.prefix();
+        trace!("Attempting to update prefixmap for {:?}", prefix);
+        let section_key = match self.section_by_prefix(&prefix) {
             Ok(sap) => sap.section_key(),
-            Err(_) => self.genesis_pk,
+            Err(_) => {
+                trace!("No key found for prefix: {:?}", prefix);
+                self.genesis_pk
+            }
         };
 
-        self.verify_with_chain_and_update(
-            signed_section_auth,
+        let res = self.verify_with_chain_and_update(
+            signed_sap,
             proof_chain,
             &SecuredLinkedList::new(section_key),
-        )
+        );
+
+        for section in self.sections.iter() {
+            let prefix = section.key();
+            trace!("Known prefix after update: {:?}", prefix);
+        }
+
+        res
     }
 
     /// Update our knowledge of a remote section's SAP only
     /// if it's verifiable with the provided proof chain and section chain.
+    /// Returns true if an udpate was made
     pub(crate) fn verify_with_chain_and_update(
         &self,
-        signed_section_auth: SectionAuth<SectionAuthorityProvider>,
+        signed_sap: SectionAuth<SectionAuthorityProvider>,
         proof_chain: &SecuredLinkedList,
         section_chain: &SecuredLinkedList,
     ) -> Result<bool> {
         // Check if SAP signature is valid
-        if !signed_section_auth.self_verify() {
+        if !signed_sap.self_verify() {
             return Err(Error::UntrustedSectionAuthProvider(format!(
                 "invalid signature: {:?}",
-                signed_section_auth.value
+                signed_sap.value
             )));
         }
 
         // Check if SAP's section key matches SAP signature's key
-        if signed_section_auth.sig.public_key
-            != signed_section_auth.value.public_key_set.public_key()
-        {
+        if signed_sap.sig.public_key != signed_sap.section_key() {
             return Err(Error::UntrustedSectionAuthProvider(format!(
                 "section key doesn't match signature's key: {:?}",
-                signed_section_auth.value
+                signed_sap.value
             )));
         }
 
@@ -173,15 +198,15 @@ impl NetworkPrefixMap {
         // as trusted before we store them in our local records.
         // Thus, we just need to check our knowledge of the remote section's key
         // is part of the proof chain received.
-        match self.sections.get(&signed_section_auth.value.prefix) {
-            Some(entry) if entry.value() == &signed_section_auth => {
+        match self.sections.get(&signed_sap.prefix()) {
+            Some(entry) if entry.value() == &signed_sap => {
                 // It's the same SAP we are already aware of
                 return Ok(false);
             }
             Some(entry) => {
                 // We are then aware of the prefix, let's just verify the new SAP can
                 // be trusted based on the SAP we aware of and the proof chain provided.
-                if !proof_chain.has_key(&entry.value().value.public_key_set.public_key()) {
+                if !proof_chain.has_key(&entry.value().section_key()) {
                     // This case may happen when both the sender and receiver is about to using
                     // a new SAP. The AE-Update was sent before sender switching to use new SAP,
                     // hence it only contains proof_chain covering the old SAP.
@@ -192,7 +217,7 @@ impl NetworkPrefixMap {
                     // avoid potential looping.
                     return Err(Error::UntrustedProofChain(format!(
                         "provided proof_chain doesn't cover the SAP's key we currently know: {:?}",
-                        signed_section_auth.value
+                        entry.value().value
                     )));
                 }
             }
@@ -202,7 +227,7 @@ impl NetworkPrefixMap {
                 if !proof_chain.check_trust(section_chain.keys()) {
                     return Err(Error::UntrustedProofChain(format!(
                         "none of the keys were found on our section chain: {:?}",
-                        signed_section_auth.value
+                        signed_sap.value
                     )));
                 }
             }
@@ -218,10 +243,11 @@ impl NetworkPrefixMap {
         }
 
         // Check the SAP's key is the last key of the proof chain
-        if proof_chain.last_key() != &signed_section_auth.value.public_key_set.public_key() {
+        if proof_chain.last_key() != &signed_sap.section_key() {
             return Err(Error::UntrustedSectionAuthProvider(format!(
-                "section key ({:?}) doesn't match proof chain last key ({:?})",
-                signed_section_auth.value.public_key_set.public_key(),
+                "section key ({:?}, from prefix {:?}) isn't in the last key in the proof chain provided. (Which ends with ({:?}))",
+                signed_sap.section_key(),
+                signed_sap.prefix(),
                 proof_chain.last_key()
             )));
         }
@@ -229,21 +255,25 @@ impl NetworkPrefixMap {
         // We can now update our knowledge of the remote section's SAP.
         // Note: we don't expect the same SAP to be found in our records
         // for the prefix since we've already checked that above.
-        let _ = self.insert(signed_section_auth);
+        let changed = self.insert(signed_sap);
 
-        Ok(true)
+        for section in self.sections.iter() {
+            let prefix = section.key();
+            trace!("Known prefix: {:?}", prefix);
+        }
+
+        Ok(changed)
     }
 
     /// Returns the known section public keys.
     pub(crate) fn section_keys(&self) -> Vec<bls::PublicKey> {
         self.sections
             .iter()
-            .map(|e| e.value().value.section_key())
+            .map(|e| e.value().section_key())
             .collect()
     }
 
-    /// Returns the section authority provider for the prefix that matches `name`,
-    /// excluding self section.
+    /// Returns the section authority provider for the prefix that matches `name`.
     pub(crate) fn section_by_name(&self, name: &XorName) -> Result<SectionAuthorityProvider> {
         self.sections
             .iter()
@@ -268,22 +298,20 @@ impl NetworkPrefixMap {
     pub(crate) fn network_stats(&self, our: &SectionAuthorityProvider) -> NetworkStats {
         // Let's compute an estimate of the total number of elders in the network
         // from the size of our routing table.
-        let section_prefixes: Vec<Prefix> = self.sections.iter().map(|e| *e.key()).collect();
-        let known_prefixes = section_prefixes.iter().chain(iter::once(&our.prefix));
+        let section_prefixes = self.sections.iter().map(|e| *e.key());
+        let known_prefixes: Vec<_> = section_prefixes.chain(iter::once(our.prefix())).collect();
 
-        let total_elders_exact = Prefix::default().is_covered_by(known_prefixes.clone());
+        let total_elders_exact = Prefix::default().is_covered_by(&known_prefixes);
 
         // Estimated fraction of the network that we have in our RT.
         // Computed as the sum of 1 / 2^(prefix.bit_count) for all known section prefixes.
         let network_fraction: f64 = known_prefixes
+            .iter()
             .map(|p| 1.0 / (p.bit_count() as f64).exp2())
             .sum();
 
-        let network_elders_count: usize = self
-            .sections
-            .iter()
-            .map(|e| e.value().value.elder_count())
-            .sum();
+        let network_elders_count: usize =
+            self.sections.iter().map(|e| e.value().elder_count()).sum();
         let total = network_elders_count as f64 / network_fraction;
 
         // `total_elders_exact` indicates whether `total_elders` is
@@ -319,7 +347,7 @@ impl NetworkPrefixMap {
                 prefix.is_covered_by(descendant_prefixes)
             };
             if is_covered {
-                let _ = self.sections.remove(&prefix);
+                let _prev = self.sections.remove(&prefix);
             }
 
             if prefix.is_empty() {
@@ -331,6 +359,33 @@ impl NetworkPrefixMap {
     }
 }
 
+impl<'de> Deserialize<'de> for NetworkPrefixMap {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        // This is easier than hand-writing an impl, not sure if there's a downside.
+        // Uses the same name in case it shows up in errors messages etc.
+        #[derive(Deserialize)]
+        struct NetworkPrefixMap {
+            sections: DashMap<Prefix, SectionAuth<crate::messaging::SectionAuthorityProvider>>,
+            genesis_pk: BlsPublicKey,
+        }
+
+        let helper = NetworkPrefixMap::deserialize(deserializer)?;
+        let sections = helper
+            .sections
+            .into_iter()
+            .map(|(k, v)| (k, v.into_authed_state()))
+            .collect();
+
+        Ok(Self {
+            sections: Arc::new(sections),
+            genesis_pk: helper.genesis_pk,
+        })
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -393,21 +448,21 @@ mod tests {
 
         let sap0 = gen_section_auth(p0)?;
 
-        let _ = map.insert(sap0.clone());
+        let _changed = map.insert(sap0.clone());
 
         // There are no matching prefixes, so return Err.
         assert!(map.section_by_name(&p1.substituted_in(rng.gen())).is_err(),);
 
         // There are no matching prefixes, so return an opposite prefix.
         assert_eq!(
-            map.closest_or_opposite(&p1.substituted_in(rng.gen()))
+            map.closest_or_opposite(&p1.substituted_in(rng.gen()), None)
                 .ok_or(Error::NoMatchingSection)?,
             sap0
         );
 
-        let _ = map.insert(sap0.clone());
+        let _changed = map.insert(sap0.clone());
         assert_eq!(
-            map.closest_or_opposite(&p1.substituted_in(rng.gen()))
+            map.closest_or_opposite(&p1.substituted_in(rng.gen()), None)
                 .ok_or(Error::NoMatchingSection)?,
             sap0
         );
@@ -464,7 +519,7 @@ mod tests {
 
         let sap0 = gen_section_auth(p0)?;
         let sap00 = gen_section_auth(p00)?;
-        let _ = map.insert(sap00.clone());
+        let _changed = map.insert(sap00.clone());
 
         assert!(!map.insert(sap0));
         assert_eq!(map.get(&p0), None);
@@ -486,9 +541,9 @@ mod tests {
         let sap1 = gen_section_auth(p1)?;
         let sap10 = gen_section_auth(p10)?;
 
-        let _ = map.insert(sap0.clone());
-        let _ = map.insert(sap1.clone());
-        let _ = map.insert(sap10.clone());
+        let _changed = map.insert(sap0.clone());
+        let _changed = map.insert(sap1.clone());
+        let _changed = map.insert(sap10.clone());
 
         assert_eq!(
             map.section_by_name(&p0.substituted_in(rng.gen()))?,
@@ -519,9 +574,9 @@ mod tests {
         let sap1 = gen_section_auth(p1)?;
         let sap10 = gen_section_auth(p10)?;
 
-        let _ = map.insert(sap0.clone());
-        let _ = map.insert(sap1.clone());
-        let _ = map.insert(sap10.clone());
+        let _changed = map.insert(sap0.clone());
+        let _changed = map.insert(sap1.clone());
+        let _changed = map.insert(sap10.clone());
 
         assert_eq!(map.section_by_prefix(&p0)?, sap0.value);
 
@@ -545,26 +600,26 @@ mod tests {
 
         let mut chain01 = chain.clone();
         let section_auth_01 = gen_section_auth(p01)?;
-        let pk01 = section_auth_01.value.public_key_set.public_key();
+        let pk01 = section_auth_01.section_key();
         let sig01 = bincode::serialize(&pk01).map(|bytes| genesis_sk.sign(&bytes))?;
         chain01.insert(&genesis_pk, pk01, sig01)?;
-        let _ = map.verify_with_chain_and_update(section_auth_01, &chain01, &chain);
+        let _updated = map.verify_with_chain_and_update(section_auth_01, &chain01, &chain);
 
         let mut chain10 = chain.clone();
         let section_auth_10 = gen_section_auth(p10)?;
-        let pk10 = section_auth_10.value.public_key_set.public_key();
+        let pk10 = section_auth_10.section_key();
         let sig10 = bincode::serialize(&pk10).map(|bytes| genesis_sk.sign(&bytes))?;
         chain10.insert(&genesis_pk, pk10, sig10)?;
-        let _ = map.verify_with_chain_and_update(section_auth_10, &chain10, &chain);
+        let _updated = map.verify_with_chain_and_update(section_auth_10, &chain10, &chain);
 
         let mut rng = rand::thread_rng();
         let n01 = p01.substituted_in(rng.gen());
         let n10 = p10.substituted_in(rng.gen());
         let n11 = p11.substituted_in(rng.gen());
 
-        assert_eq!(map.closest(&n01).map(|sap| sap.value.prefix), Some(p01));
-        assert_eq!(map.closest(&n10).map(|sap| sap.value.prefix), Some(p10));
-        assert_eq!(map.closest(&n11).map(|sap| sap.value.prefix), Some(p10));
+        assert_eq!(map.closest(&n01, None).map(|sap| sap.prefix()), Some(p01));
+        assert_eq!(map.closest(&n10, None).map(|sap| sap.prefix()), Some(p10));
+        assert_eq!(map.closest(&n11, None).map(|sap| sap.prefix()), Some(p10));
 
         Ok(())
     }
