@@ -6,36 +6,44 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-use super::{msg_count::MsgCount, BackPressure};
+use super::{connected_peers::ConnectedPeers, msg_count::MsgCount, BackPressure};
 use crate::messaging::{system::LoadReport, WireMsg};
-use crate::routing::error::{Error, Result};
+use crate::routing::{
+    error::{Error, Result},
+    log_markers::LogMarker,
+    Peer,
+};
 use bytes::Bytes;
 use futures::{
     future::{FutureExt, TryFutureExt},
     stream::{FuturesUnordered, StreamExt},
 };
 use qp2p::Endpoint;
-use std::net::SocketAddr;
+use std::{future, net::SocketAddr};
 use tokio::{sync::mpsc, task};
+use tracing::Instrument;
 use xor_name::XorName;
 
 // Communication component of the node to interact with other nodes.
 #[derive(Clone)]
 pub(crate) struct Comm {
-    endpoint: Endpoint<XorName>,
+    endpoint: Endpoint,
     event_tx: mpsc::Sender<ConnectionEvent>,
     msg_count: MsgCount,
     back_pressure: BackPressure,
+    connected_peers: ConnectedPeers,
 }
 
 impl Drop for Comm {
     fn drop(&mut self) {
         // Close all existing connections and stop accepting new ones.
+        // FIXME: this may be broken – `Comm` is clone, so this will break any clones?
         self.endpoint.close();
     }
 }
 
 impl Comm {
+    #[tracing::instrument(skip_all)]
     pub(crate) async fn new(
         local_addr: SocketAddr,
         config: qp2p::Config,
@@ -49,12 +57,17 @@ impl Comm {
             Endpoint::new(local_addr, Default::default(), config).await?;
 
         let msg_count = MsgCount::new();
+        let connected_peers = ConnectedPeers::default();
 
-        let _ = task::spawn(handle_incoming_connections(
-            incoming_connections,
-            event_tx.clone(),
-            msg_count.clone(),
-        ));
+        let _handle = task::spawn(
+            handle_incoming_connections(
+                incoming_connections,
+                event_tx.clone(),
+                msg_count.clone(),
+                connected_peers.clone(),
+            )
+            .in_current_span(),
+        );
 
         let back_pressure = BackPressure::new();
 
@@ -63,9 +76,11 @@ impl Comm {
             event_tx,
             msg_count,
             back_pressure,
+            connected_peers,
         })
     }
 
+    #[tracing::instrument(skip(local_addr, config, event_tx))]
     pub(crate) async fn bootstrap(
         local_addr: SocketAddr,
         bootstrap_nodes: &[SocketAddr],
@@ -79,21 +94,29 @@ impl Comm {
         let (bootstrap_peer, peer_incoming) = bootstrap_peer.ok_or(Error::BootstrapFailed)?;
 
         let msg_count = MsgCount::new();
+        let connected_peers = ConnectedPeers::default();
 
-        let _ = task::spawn(handle_incoming_connections(
-            incoming_connections,
-            event_tx.clone(),
-            msg_count.clone(),
-        ));
-
-        if let Some(incoming_messages) = peer_incoming {
-            let _ = task::spawn(handle_incoming_messages(
-                bootstrap_peer.remote_address(),
-                incoming_messages,
+        let _handle = task::spawn(
+            handle_incoming_connections(
+                incoming_connections,
                 event_tx.clone(),
                 msg_count.clone(),
-            ));
-        }
+                connected_peers.clone(),
+            )
+            .in_current_span(),
+        );
+
+        let _ = task::spawn(
+            handle_incoming_messages(
+                bootstrap_peer.id(),
+                bootstrap_peer.remote_address(),
+                peer_incoming,
+                event_tx.clone(),
+                msg_count.clone(),
+                connected_peers.clone(),
+            )
+            .in_current_span(),
+        );
 
         Ok((
             Self {
@@ -101,6 +124,7 @@ impl Comm {
                 event_tx,
                 msg_count,
                 back_pressure: BackPressure::new(),
+                connected_peers,
             },
             bootstrap_peer.remote_address(),
         ))
@@ -111,40 +135,40 @@ impl Comm {
     }
 
     /// Get the connection ID (XorName) of an existing connection with the provided socket address
-    pub(crate) async fn get_connection_id(&self, addr: &SocketAddr) -> Option<XorName> {
-        self.endpoint
-            .get_connection_by_addr(addr)
-            .await
-            .map(|connection| connection.id())
+    pub(crate) async fn get_peer_connection_id(&self, address: &SocketAddr) -> Option<XorName> {
+        let peer = self.connected_peers.get_by_address(address).await?;
+        Some(peer.id())
     }
 
     /// Get the SocketAddr of a connection using the connection ID (XorName)
-    pub(crate) async fn get_socket_addr_by_id(&self, xorname: &XorName) -> Option<SocketAddr> {
-        self.endpoint
-            .get_connection_by_id(xorname)
-            .await
-            .map(|connection| connection.remote_address())
+    pub(crate) async fn get_peer_address(&self, connection_id: &XorName) -> Option<SocketAddr> {
+        let peer = self.connected_peers.get_by_id(connection_id).await?;
+        Some(peer.address())
     }
 
     /// Sends a message on an existing connection. If no such connection exists, returns an error.
     pub(crate) async fn send_on_existing_connection(
         &self,
-        recipients: &[(XorName, SocketAddr)],
+        recipients: &[Peer],
         mut wire_msg: WireMsg,
     ) -> Result<(), Error> {
         trace!("Sending msg on existing connection to {:?}", recipients);
-        for (name, addr) in recipients {
-            wire_msg.set_dst_xorname(*name);
+        for recipient in recipients {
+            let name = recipient.name();
+            let addr = recipient.addr();
+
+            wire_msg.set_dst_xorname(name);
 
             let bytes = wire_msg.serialize()?;
             let priority = wire_msg.msg_kind().priority();
-            let retries = self.back_pressure.get(addr).await; // TODO: more laid back retries with lower priority, more aggressive with higher
+            let retries = self.back_pressure.get(&addr).await; // TODO: more laid back retries with lower priority, more aggressive with higher
 
-            self.endpoint
-                .get_connection_by_addr(addr)
+            self.connected_peers
+                .get_by_address(&addr)
                 .map(|res| res.ok_or(None))
-                .and_then(|connection| async move {
-                    connection
+                .and_then(|client| async move {
+                    client
+                        .connection()
                         .send_with(bytes, priority, Some(&retries))
                         .await
                         .map_err(Some)
@@ -158,11 +182,11 @@ impl Comm {
                         name,
                         err
                     );
-                    Error::FailedSend(*addr, *name)
+                    Error::FailedSend(addr, name)
                 })?;
 
             // count outgoing msgs..
-            self.msg_count.increase_outgoing(*addr);
+            self.msg_count.increase_outgoing(addr);
         }
 
         Ok(())
@@ -176,7 +200,7 @@ impl Comm {
         };
 
         let connectivity_endpoint =
-            Endpoint::<XorName>::new_client((self.endpoint.local_addr().ip(), 0), qp2p_config)?;
+            Endpoint::new_client((self.endpoint.local_addr().ip(), 0), qp2p_config)?;
 
         let result = connectivity_endpoint
             .is_reachable(peer)
@@ -201,9 +225,14 @@ impl Comm {
     /// `SendStatus::MinDeliveryGroupSizeReached` or `SendStatus::MinDeliveryGroupSizeFailed` depending
     /// on if the minimum delivery group size is met or not. The failed recipients are sent along
     /// with the status. It returns a `SendStatus::AllRecipients` if message is sent to all the recipients.
-    pub(crate) async fn send(
+    #[allow(clippy::needless_lifetimes)]
+    // ^ this is firing a false positive here
+    // we need an explicit lifetime for the compiler to see
+    // that `recipient` lives long enough in the closure
+    #[tracing::instrument(skip(self))]
+    pub(crate) async fn send<'r>(
         &self,
-        recipients: &[(XorName, SocketAddr)],
+        recipients: &'r [Peer],
         delivery_group_size: usize,
         wire_msg: WireMsg,
     ) -> Result<SendStatus> {
@@ -235,63 +264,78 @@ impl Comm {
         // Run all the sends concurrently (using `FuturesUnordered`). If any of them fails, pick
         // the next recipient and try to send to them. Proceed until the needed number of sends
         // succeeds or if there are no more recipients to pick.
-        let send = |recipient: (XorName, SocketAddr), msg_bytes: Bytes| async move {
-            trace!(
-                "Sending message ({} bytes, msg_id: {:?}) to {} of delivery group size {}",
-                msg_bytes.len(),
-                msg_id,
-                recipient.1,
-                delivery_group_size,
-            );
+        let send = |recipient: &'r Peer, msg_bytes: Bytes| {
+            async move {
+                trace!(
+                    "Sending message ({} bytes, msg_id: {:?}) to {} of delivery group size {}",
+                    msg_bytes.len(),
+                    msg_id,
+                    recipient,
+                    delivery_group_size,
+                );
 
-            let retries = self.back_pressure.get(&recipient.1).await; // TODO: more laid back retries with lower priority, more aggressive with higher
+                let retries = self.back_pressure.get(&recipient.addr()).await; // TODO: more laid back retries with lower priority, more aggressive with higher
 
-            let result = self
-                .endpoint
-                .connect_to(&recipient.1)
-                .err_into()
-                .and_then(|(connection, connection_incoming)| async move {
-                    if let Some(connection_incoming) = connection_incoming {
-                        let _ = task::spawn(handle_incoming_messages(
-                            connection.remote_address(),
-                            connection_incoming,
-                            self.event_tx.clone(),
-                            self.msg_count.clone(),
-                        ));
-                    }
-                    connection
-                        .send_with(msg_bytes, priority, Some(&retries))
+                let connection = if let Some(connection) =
+                    self.connected_peers.get_by_address(&recipient.addr()).await
+                {
+                    Ok(connection.connection().clone())
+                } else {
+                    self.endpoint
+                        .connect_to(&recipient.addr())
+                        .and_then(|(connection, connection_incoming)| async move {
+                            self.connected_peers.insert(connection.clone()).await;
+                            let _ = task::spawn(handle_incoming_messages(
+                                connection.id(),
+                                connection.remote_address(),
+                                connection_incoming,
+                                self.event_tx.clone(),
+                                self.msg_count.clone(),
+                                self.connected_peers.clone(),
+                            ));
+                            Ok(connection)
+                        })
                         .await
-                })
-                .await
-                .map_err(|err| match err {
-                    qp2p::SendError::ConnectionLost(qp2p::ConnectionError::Closed(
-                        qp2p::Close::Local,
-                    )) => Error::ConnectionClosed,
-                    _ => {
-                        trace!("during sending, received error {:?}", err);
-                        err.into()
-                    }
-                });
+                };
 
-            (result, recipient.1)
+                let result = future::ready(connection)
+                    .err_into()
+                    .and_then(|connection| async move {
+                        connection
+                            .send_with(msg_bytes, priority, Some(&retries))
+                            .await
+                    })
+                    .await
+                    .map_err(|err| match err {
+                        qp2p::SendError::ConnectionLost(qp2p::ConnectionError::Closed(
+                            qp2p::Close::Local,
+                        )) => Error::ConnectionClosed,
+                        _ => {
+                            warn!("during sending, received error {:?}", err);
+                            err.into()
+                        }
+                    });
+
+                (result, recipient)
+            }
+            .in_current_span()
         };
 
         let mut tasks: FuturesUnordered<_> = recipients[0..delivery_group_size]
             .iter()
-            .map(|(name, recipient)| send((*name, *recipient), msg_bytes.clone()))
+            .map(|recipient| send(recipient, msg_bytes.clone()))
             .collect();
 
         let mut next = delivery_group_size;
         let mut successes = 0;
         let mut failed_recipients = vec![];
 
-        while let Some((result, addr)) = tasks.next().await {
+        while let Some((result, recipient)) = tasks.next().await {
             match result {
                 Ok(()) => {
                     successes += 1;
                     // count outgoing msgs..
-                    self.msg_count.increase_outgoing(addr);
+                    self.msg_count.increase_outgoing(recipient.addr());
                 }
                 Err(Error::ConnectionClosed) => {
                     // The connection was closed by us which means
@@ -299,10 +343,10 @@ impl Comm {
                     return Err(Error::ConnectionClosed);
                 }
                 Err(_) => {
-                    failed_recipients.push(addr);
+                    failed_recipients.push(recipient.clone());
 
                     if next < recipients.len() {
-                        tasks.push(send(recipients[next], msg_bytes.clone()));
+                        tasks.push(send(&recipients[next], msg_bytes.clone()));
                         next += 1;
                     }
                 }
@@ -352,31 +396,47 @@ pub(crate) enum ConnectionEvent {
     Received((SocketAddr, Bytes)),
 }
 
+#[tracing::instrument(skip_all)]
 async fn handle_incoming_connections(
-    mut incoming_connections: qp2p::IncomingConnections<XorName>,
+    mut incoming_connections: qp2p::IncomingConnections,
     event_tx: mpsc::Sender<ConnectionEvent>,
     msg_count: MsgCount,
+    connected_peers: ConnectedPeers,
 ) {
     while let Some((connection, connection_incoming)) = incoming_connections.next().await {
-        let _ = task::spawn(handle_incoming_messages(
-            connection.remote_address(),
-            connection_incoming,
-            event_tx.clone(),
-            msg_count.clone(),
-        ));
+        let connection_id = connection.id();
+        let src = connection.remote_address();
+        connected_peers.insert(connection).await;
+
+        let _ = task::spawn(
+            handle_incoming_messages(
+                connection_id,
+                src,
+                connection_incoming,
+                event_tx.clone(),
+                msg_count.clone(),
+                connected_peers.clone(),
+            )
+            .in_current_span(),
+        );
     }
 }
 
+#[tracing::instrument(skip(incoming_msgs, event_tx, msg_count, connected_peers))]
 async fn handle_incoming_messages(
+    connection_id: usize,
     src: SocketAddr,
-    mut incoming_msgs: qp2p::ConnectionIncoming<XorName>,
+    mut incoming_msgs: qp2p::ConnectionIncoming,
     event_tx: mpsc::Sender<ConnectionEvent>,
     msg_count: MsgCount,
+    connected_peers: ConnectedPeers,
 ) {
+    trace!(%connection_id, %src, "{}", LogMarker::ConnectionOpened);
+
     while let Some(result) = incoming_msgs.next().await.transpose() {
         match result {
             Ok(msg) => {
-                let _ = event_tx.send(ConnectionEvent::Received((src, msg))).await;
+                let _send_res = event_tx.send(ConnectionEvent::Received((src, msg))).await;
                 // count incoming msgs..
                 msg_count.increase_incoming(src);
             }
@@ -386,21 +446,28 @@ async fn handle_incoming_messages(
             }
         }
     }
+
+    // remove the connection once we notice it end
+    connected_peers.remove_by_address(&src).await;
+
+    trace!(%connection_id, %src, "{}", LogMarker::ConnectionClosed);
 }
 
 /// Returns the status of the send operation.
 #[derive(Debug, Clone)]
 pub(crate) enum SendStatus {
     AllRecipients,
-    MinDeliveryGroupSizeReached(Vec<SocketAddr>),
-    MinDeliveryGroupSizeFailed(Vec<SocketAddr>),
+    MinDeliveryGroupSizeReached(Vec<Peer>),
+    MinDeliveryGroupSizeFailed(Vec<Peer>),
 }
 
 #[cfg(test)]
 mod tests {
     use super::*;
+    use crate::messaging::data::{DataQuery, ServiceMsg};
     use crate::messaging::{DstLocation, MessageId, MsgKind, ServiceAuth};
-    use crate::types::Keypair;
+    use crate::routing::Peer;
+    use crate::types::{ChunkAddress, Keypair};
     use assert_matches::assert_matches;
     use eyre::Result;
     use futures::future;
@@ -416,26 +483,22 @@ mod tests {
         let (tx, _rx) = mpsc::channel(1);
         let comm = Comm::new(local_addr(), Config::default(), tx).await?;
 
-        let mut peer0 = Peer::new().await?;
-        let mut peer1 = Peer::new().await?;
+        let (peer0, mut rx0) = new_peer().await?;
+        let (peer1, mut rx1) = new_peer().await?;
 
         let original_message = new_test_message()?;
 
         let status = comm
-            .send(
-                &[(peer0.name, peer0.addr), (peer1.name, peer1.addr)],
-                2,
-                original_message.clone(),
-            )
+            .send(&[peer0, peer1], 2, original_message.clone())
             .await?;
 
         assert_matches!(status, SendStatus::AllRecipients);
 
-        if let Some(bytes) = peer0.rx.recv().await {
+        if let Some(bytes) = rx0.recv().await {
             assert_eq!(WireMsg::from(bytes)?, original_message.clone());
         }
 
-        if let Some(bytes) = peer1.rx.recv().await {
+        if let Some(bytes) = rx1.recv().await {
             assert_eq!(WireMsg::from(bytes)?, original_message);
         }
 
@@ -447,25 +510,21 @@ mod tests {
         let (tx, _rx) = mpsc::channel(1);
         let comm = Comm::new(local_addr(), Config::default(), tx).await?;
 
-        let mut peer0 = Peer::new().await?;
-        let mut peer1 = Peer::new().await?;
+        let (peer0, mut rx0) = new_peer().await?;
+        let (peer1, mut rx1) = new_peer().await?;
 
         let original_message = new_test_message()?;
         let status = comm
-            .send(
-                &[(peer0.name, peer0.addr), (peer1.name, peer1.addr)],
-                1,
-                original_message.clone(),
-            )
+            .send(&[peer0, peer1], 1, original_message.clone())
             .await?;
 
         assert_matches!(status, SendStatus::AllRecipients);
 
-        if let Some(bytes) = peer0.rx.recv().await {
+        if let Some(bytes) = rx0.recv().await {
             assert_eq!(WireMsg::from(bytes)?, original_message);
         }
 
-        assert!(time::timeout(TIMEOUT, peer1.rx.recv())
+        assert!(time::timeout(TIMEOUT, rx1.recv())
             .await
             .unwrap_or_default()
             .is_none());
@@ -486,11 +545,10 @@ mod tests {
             tx,
         )
         .await?;
-        let invalid_addr = get_invalid_addr().await?;
+        let invalid_peer = get_invalid_peer().await?;
+        let invalid_addr = invalid_peer.addr();
 
-        let status = comm
-            .send(&[(XorName::random(), invalid_addr)], 1, new_test_message()?)
-            .await?;
+        let status = comm.send(&[invalid_peer], 1, new_test_message()?).await?;
 
         assert_matches!(
             &status,
@@ -512,23 +570,18 @@ mod tests {
             tx,
         )
         .await?;
-        let mut peer = Peer::new().await?;
-        let invalid_addr = get_invalid_addr().await?;
-        let name = XorName::random();
+        let (peer, mut rx) = new_peer().await?;
+        let invalid_peer = get_invalid_peer().await?;
 
         let message = new_test_message()?;
         let status = comm
-            .send(
-                &[(name, invalid_addr), (peer.name, peer.addr)],
-                1,
-                message.clone(),
-            )
+            .send(&[invalid_peer.clone(), peer], 1, message.clone())
             .await?;
         assert_matches!(status, SendStatus::MinDeliveryGroupSizeReached(failed_recipients) => {
-            assert_eq!(&failed_recipients, &[invalid_addr])
+            assert_eq!(&failed_recipients, &[invalid_peer])
         });
 
-        if let Some(bytes) = peer.rx.recv().await {
+        if let Some(bytes) = rx.recv().await {
             assert_eq!(WireMsg::from(bytes)?, message);
         }
         Ok(())
@@ -546,25 +599,20 @@ mod tests {
             tx,
         )
         .await?;
-        let mut peer = Peer::new().await?;
-        let invalid_addr = get_invalid_addr().await?;
-        let name = XorName::random();
+        let (peer, mut rx) = new_peer().await?;
+        let invalid_peer = get_invalid_peer().await?;
 
         let message = new_test_message()?;
         let status = comm
-            .send(
-                &[(name, invalid_addr), (peer.name, peer.addr)],
-                2,
-                message.clone(),
-            )
+            .send(&[invalid_peer.clone(), peer], 2, message.clone())
             .await?;
 
         assert_matches!(
             status,
-            SendStatus::MinDeliveryGroupSizeFailed(_) => vec![invalid_addr]
+            SendStatus::MinDeliveryGroupSizeFailed(_) => vec![invalid_peer]
         );
 
-        if let Some(bytes) = peer.rx.recv().await {
+        if let Some(bytes) = rx.recv().await {
             assert_eq!(WireMsg::from(bytes)?, message);
         }
         Ok(())
@@ -576,13 +624,13 @@ mod tests {
         let send_comm = Comm::new(local_addr(), Config::default(), tx).await?;
 
         let (recv_endpoint, mut incoming_connections, _) =
-            Endpoint::<XorName>::new(local_addr(), &[], Config::default()).await?;
+            Endpoint::new(local_addr(), &[], Config::default()).await?;
         let recv_addr = recv_endpoint.public_addr();
         let name = XorName::random();
 
         let msg0 = new_test_message()?;
         let status = send_comm
-            .send(&[(name, recv_addr)], 1, msg0.clone())
+            .send(&[Peer::new(name, recv_addr)], 1, msg0.clone())
             .await?;
         assert_matches!(status, SendStatus::AllRecipients);
 
@@ -590,22 +638,20 @@ mod tests {
 
         // Receive one message and disconnect from the peer
         {
-            if let Some((connection, mut incoming_msgs)) = incoming_connections.next().await {
+            if let Some((_, mut incoming_msgs)) = incoming_connections.next().await {
                 if let Some(msg) = time::timeout(TIMEOUT, incoming_msgs.next()).await?? {
                     assert_eq!(WireMsg::from(msg)?, msg0);
                     msg0_received = true;
-                    recv_endpoint
-                        .disconnect_from(&connection.remote_address())
-                        .await;
-                    tokio::time::sleep(Duration::from_secs(1)).await;
                 }
+
+                // connection dropped here
             }
             assert!(msg0_received);
         }
 
         let msg1 = new_test_message()?;
         let status = send_comm
-            .send(&[(name, recv_addr)], 1, msg1.clone())
+            .send(&[Peer::new(name, recv_addr)], 1, msg1.clone())
             .await?;
         assert_matches!(status, SendStatus::AllRecipients);
 
@@ -634,7 +680,11 @@ mod tests {
 
         // Send a message to establish the connection
         let status = comm1
-            .send(&[(XorName::random(), addr0)], 1, new_test_message()?)
+            .send(
+                &[Peer::new(XorName::random(), addr0)],
+                1,
+                new_test_message()?,
+            )
             .await?;
         assert_matches!(status, SendStatus::AllRecipients);
 
@@ -647,6 +697,49 @@ mod tests {
         Ok(())
     }
 
+    #[tokio::test(flavor = "multi_thread")]
+    async fn connected_peers() -> Result<()> {
+        let (node_tx, mut node_rx) = mpsc::channel(1);
+        let node_comm = Comm::new(local_addr(), Config::default(), node_tx).await?;
+        let node_addr = node_comm.our_connection_info();
+
+        let (client_tx, mut client_rx) = mpsc::channel(1);
+        let client_comm = Comm::new(local_addr(), Config::default(), client_tx).await?;
+        let client_addr = client_comm.our_connection_info();
+
+        // Establish a connection by sending a message
+        let status = client_comm
+            .send(
+                &[Peer::new(XorName::random(), node_addr)],
+                1,
+                new_test_message()?,
+            )
+            .await?;
+        assert_matches!(status, SendStatus::AllRecipients);
+
+        // We should have recorded the connection
+        assert_matches!(node_rx.recv().await, Some(ConnectionEvent::Received(_)));
+        assert!(
+            node_comm
+                .get_peer_connection_id(&client_addr)
+                .await
+                .is_some(),
+            "did not find expected connection"
+        );
+
+        // We can reply to the client over the existing connection
+        node_comm
+            .send_on_existing_connection(
+                &[Peer::new(XorName::random(), client_addr)],
+                new_test_message()?,
+            )
+            .await?;
+
+        assert_matches!(client_rx.recv().await, Some(ConnectionEvent::Received(_)));
+
+        Ok(())
+    }
+
     fn new_test_message() -> Result<WireMsg> {
         let dst_location = DstLocation::Node {
             name: XorName::random(),
@@ -656,7 +749,9 @@ mod tests {
         let mut rng = OsRng;
         let src_keypair = Keypair::new_ed25519(&mut rng);
 
-        let payload = WireMsg::serialize_msg_payload(&"test_string".to_string())?;
+        let payload = WireMsg::serialize_msg_payload(&ServiceMsg::Query(DataQuery::GetChunk(
+            ChunkAddress(XorName::random()),
+        )))?;
         let auth = ServiceAuth {
             public_key: src_keypair.public_key(),
             signature: src_keypair.sign(&payload),
@@ -672,48 +767,36 @@ mod tests {
         Ok(wire_msg)
     }
 
-    struct Peer {
-        addr: SocketAddr,
-        name: XorName,
-        rx: mpsc::Receiver<Bytes>,
-    }
+    async fn new_peer() -> Result<(Peer, mpsc::Receiver<Bytes>)> {
+        let (endpoint, mut incoming_connections, _) =
+            Endpoint::new(local_addr(), &[], Config::default()).await?;
+        let addr = endpoint.public_addr();
 
-    impl Peer {
-        async fn new() -> Result<Self> {
-            let (endpoint, mut incoming_connections, _) =
-                Endpoint::<XorName>::new(local_addr(), &[], Config::default()).await?;
-            let addr = endpoint.public_addr();
+        let (tx, rx) = mpsc::channel(1);
 
-            let (tx, rx) = mpsc::channel(1);
-
-            let _ = tokio::spawn(async move {
-                while let Some((_, mut incoming_messages)) = incoming_connections.next().await {
-                    while let Ok(Some(msg)) = incoming_messages.next().await {
-                        let _ = tx.send(msg).await;
-                    }
+        let _handle = tokio::spawn(async move {
+            while let Some((_, mut incoming_messages)) = incoming_connections.next().await {
+                while let Ok(Some(msg)) = incoming_messages.next().await {
+                    let _ = tx.send(msg).await;
                 }
-            });
+            }
+        });
 
-            Ok(Self {
-                addr,
-                rx,
-                name: XorName::random(),
-            })
-        }
+        Ok((Peer::new(XorName::random(), addr), rx))
     }
 
-    async fn get_invalid_addr() -> Result<SocketAddr> {
+    async fn get_invalid_peer() -> Result<Peer> {
         let socket = UdpSocket::bind((Ipv4Addr::LOCALHOST, 0)).await?;
         let addr = socket.local_addr()?;
 
         // Keep the socket alive to keep the address bound, but don't read/write to it so any
         // attempt to connect to it will fail.
-        let _ = tokio::spawn(async move {
+        let _handle = tokio::spawn(async move {
             future::pending::<()>().await;
             let _ = socket;
         });
 
-        Ok(addr)
+        Ok(Peer::new(XorName::random(), addr))
     }
 
     fn local_addr() -> SocketAddr {
