@@ -7,8 +7,8 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use crate::messaging::{
-    system::{DkgFailureSig, DkgFailureSigSet, DkgSessionId, ElderCandidates, SystemMsg},
-    DstLocation, SectionAuthorityProvider, WireMsg,
+    system::{DkgFailureSig, DkgFailureSigSet, DkgSessionId, SystemMsg},
+    DstLocation, WireMsg,
 };
 use crate::routing::{
     dkg::dkg_msgs_utils::{DkgFailureSigSetUtils, DkgFailureSigUtils},
@@ -16,10 +16,10 @@ use crate::routing::{
     error::Result,
     log_markers::LogMarker,
     messages::WireMsgUtils,
+    network_knowledge::{ElderCandidates, SectionAuthorityProvider, SectionKeyShare},
     node::Node,
     routing_api::command::{next_timer_token, Command},
-    section::SectionKeyShare,
-    SectionAuthorityProviderUtils,
+    Peer,
 };
 use crate::types::PublicKey;
 use bls::PublicKey as BlsPublicKey;
@@ -28,13 +28,12 @@ use itertools::Itertools;
 use std::{
     collections::{BTreeSet, VecDeque},
     iter, mem,
-    net::SocketAddr,
     time::Duration,
 };
 use xor_name::XorName;
 
 // Interval to progress DKG timed phase
-const DKG_PROGRESS_INTERVAL: Duration = Duration::from_secs(30);
+const DKG_PROGRESS_INTERVAL: Duration = Duration::from_secs(3);
 
 const BACKLOG_CAPACITY: usize = 100;
 
@@ -83,13 +82,11 @@ impl Session {
         Ok(commands)
     }
 
-    fn recipients(&self) -> Vec<(XorName, SocketAddr)> {
+    fn recipients(&self) -> Vec<Peer> {
         self.elder_candidates
-            .elders
-            .iter()
+            .elders()
             .enumerate()
-            .filter(|(index, _)| *index != self.participant_index)
-            .map(|(_, (name, addr))| (*name, *addr))
+            .filter_map(|(index, peer)| (index != self.participant_index).then(|| peer.clone()))
             .collect()
     }
 
@@ -174,7 +171,7 @@ impl Session {
         section_pk: BlsPublicKey,
     ) -> Result<Vec<Command>> {
         if self.complete {
-            trace!("DKG check: complete");
+            trace!("{} {:?}", LogMarker::DkgSessionAlreadyCompleted, session_id);
             return Ok(vec![]);
         }
 
@@ -190,7 +187,11 @@ impl Session {
         };
 
         // Less than 100% participation
-        if !participants.iter().eq(self.elder_candidates.elders.keys()) {
+        if !participants
+            .iter()
+            .copied()
+            .eq(self.elder_candidates.names())
+        {
             trace!(
                 "DKG failed due to unexpected participants for {:?}: {:?}",
                 self.elder_candidates,
@@ -199,15 +200,8 @@ impl Session {
 
             let failed_participants: BTreeSet<_> = self
                 .elder_candidates
-                .elders
-                .keys()
-                .filter_map(|elder| {
-                    if !participants.contains(elder) {
-                        Some(*elder)
-                    } else {
-                        None
-                    }
-                })
+                .names()
+                .filter(|elder| !participants.contains(elder))
                 .collect();
 
             return self.report_failure(node, session_id, failed_participants, section_pk);
@@ -230,7 +224,8 @@ impl Session {
         }
 
         trace!(
-            "DKG complete for {:?}: {:?}",
+            "{} {:?}: {:?}",
+            LogMarker::DkgSessionComplete,
             self.elder_candidates,
             outcome.public_key_set.public_key()
         );
@@ -303,8 +298,7 @@ impl Session {
     ) -> Option<Command> {
         if !self
             .elder_candidates
-            .elders
-            .contains_key(&ed25519::name(&signed.public_key))
+            .contains(&ed25519::name(&signed.public_key))
         {
             return None;
         }
@@ -348,7 +342,8 @@ impl Backlog {
 
     pub(crate) fn push(&mut self, session_id: DkgSessionId, message: DkgMessage) {
         if self.0.len() == self.0.capacity() {
-            let _ = self.0.pop_front();
+            let _prev = self.0.pop_front();
+            trace!("Backlog removed {:?} due to capacity", _prev);
         }
 
         self.0.push_back((session_id, message))
@@ -383,15 +378,14 @@ mod tests {
     use crate::messaging::MessageType;
     use crate::routing::{
         dkg::voter::DkgVoter, dkg::DkgSessionIdUtils, ed25519,
-        node::test_utils::arbitrary_unique_nodes, node::Node,
-        section::section_authority_provider::ElderCandidatesUtils, section::test_utils::gen_addr,
-        ELDER_SIZE, MIN_ADULT_AGE,
+        network_knowledge::test_utils::gen_addr, node::test_utils::arbitrary_unique_nodes,
+        node::Node, ELDER_SIZE, MIN_ADULT_AGE,
     };
     use assert_matches::assert_matches;
     use eyre::{bail, ContextCompat, Result};
     use proptest::prelude::*;
     use rand::{rngs::SmallRng, SeedableRng};
-    use std::{collections::HashMap, iter};
+    use std::{collections::HashMap, iter, net::SocketAddr};
     use xor_name::Prefix;
 
     #[tokio::test]
@@ -405,7 +399,7 @@ mod tests {
             ed25519::gen_keypair(&Prefix::default().range_inclusive(), MIN_ADULT_AGE),
             gen_addr(),
         );
-        let elder_candidates = ElderCandidates::new(iter::once(node.peer()), Prefix::default());
+        let elder_candidates = ElderCandidates::new(Prefix::default(), iter::once(node.peer()));
         let session_id = DkgSessionId::new(&elder_candidates, 0);
 
         let commands = voter
@@ -433,7 +427,7 @@ mod tests {
         let mut messages = Vec::new();
 
         let elder_candidates =
-            ElderCandidates::new(nodes.iter().map(Node::peer), Prefix::default());
+            ElderCandidates::new(Prefix::default(), nodes.iter().map(Node::peer));
         let session_id = DkgSessionId::new(&elder_candidates, 0);
 
         let mut actors: HashMap<_, _> = nodes
@@ -522,7 +516,7 @@ mod tests {
                         assert_eq!(session_id, *expected_dkg_key);
                         Ok(recipients
                             .into_iter()
-                            .map(|addr| (addr.1, message.clone()))
+                            .map(|peer| (peer.addr(), message.clone()))
                             .collect())
                     }
                     other_message => bail!("Unexpected message: {:?}", other_message),
