@@ -6,18 +6,15 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-use crate::messaging::{
-    system::{DkgFailureSig, DkgFailureSigSet, DkgSessionId, ElderCandidates},
-    SectionAuthorityProvider,
-};
+use crate::messaging::system::{DkgFailureSig, DkgFailureSigSet, DkgSessionId};
 use crate::routing::{
     dkg::session::{Backlog, Session},
     ed25519,
     error::Result,
+    network_knowledge::{ElderCandidates, SectionAuthorityProvider, SectionKeyShare},
     node::Node,
     routing_api::command::Command,
-    section::{ElderCandidatesUtils, SectionKeyShare},
-    supermajority, SectionAuthorityProviderUtils,
+    supermajority,
 };
 use bls::PublicKey as BlsPublicKey;
 use bls_dkg::key_gen::{message::Message as DkgMessage, KeyGen};
@@ -67,28 +64,29 @@ impl DkgVoter {
     pub(crate) async fn start(
         &self,
         node: &Node,
-        dkg_key: DkgSessionId,
+        session_id: DkgSessionId,
         elder_candidates: ElderCandidates,
         section_pk: BlsPublicKey,
     ) -> Result<Vec<Command>> {
-        if self.sessions.contains_key(&dkg_key) {
+        if self.sessions.contains_key(&session_id) {
             trace!("DKG already in progress for {:?}", elder_candidates);
             return Ok(vec![]);
         }
 
         let name = ed25519::name(&node.keypair.public);
-        let participant_index = if let Some(index) = elder_candidates.position(&name) {
-            index
-        } else {
-            error!(
-                "DKG failed to start for {:?}: {} is not a participant",
-                elder_candidates, name
-            );
-            return Ok(vec![]);
-        };
+        let participant_index =
+            if let Some(index) = elder_candidates.names().position(|n| n == name) {
+                index
+            } else {
+                error!(
+                    "DKG failed to start for {:?}: {} is not a participant",
+                    elder_candidates, name
+                );
+                return Ok(vec![]);
+            };
 
         // Special case: only one participant.
-        if elder_candidates.elders.len() == 1 {
+        if elder_candidates.len() == 1 {
             let secret_key_set = bls::SecretKeySet::random(0, &mut rand::thread_rng());
             let section_auth = SectionAuthorityProvider::from_elder_candidates(
                 elder_candidates,
@@ -104,8 +102,8 @@ impl DkgVoter {
             }]);
         }
 
-        let threshold = supermajority(elder_candidates.elders.len()) - 1;
-        let participants = elder_candidates.elders.keys().copied().collect();
+        let threshold = supermajority(elder_candidates.len()) - 1;
+        let participants = elder_candidates.names().collect();
 
         match KeyGen::initialize(name, threshold, participants) {
             Ok((key_gen, message)) => {
@@ -121,19 +119,28 @@ impl DkgVoter {
                 };
 
                 let mut commands = vec![];
-                commands.extend(session.broadcast(node, &dkg_key, message, section_pk)?);
+                commands.extend(session.broadcast(node, &session_id, message, section_pk)?);
 
-                for message in self.backlog.write().await.take(&dkg_key).into_iter() {
-                    commands.extend(session.process_message(node, &dkg_key, message, section_pk)?);
+                trace!("Start draining backlog");
+
+                for message in self.backlog.write().await.take(&session_id).into_iter() {
+                    commands.extend(session.process_message(
+                        node,
+                        &session_id,
+                        message,
+                        section_pk,
+                    )?);
                 }
 
-                let _ = self.sessions.insert(dkg_key, session);
+                trace!("Draining backlog completed");
+
+                let _prev = self.sessions.insert(session_id, session);
 
                 // Remove uneeded old sessions.
-                self.sessions.retain(|existing_dkg_key, _| {
-                    existing_dkg_key.generation >= dkg_key.generation
+                self.sessions.retain(|existing_session_id, _| {
+                    existing_session_id.generation >= session_id.generation
                 });
-                self.backlog.write().await.prune(&dkg_key);
+                self.backlog.write().await.prune(&session_id);
 
                 Ok(commands)
             }
@@ -156,8 +163,8 @@ impl DkgVoter {
             let session = ref_mut_multi.value();
             session.timer_token() == timer_token
         }) {
-            let (dkg_key, session) = ref_mut_multi.pair_mut();
-            session.handle_timeout(node, dkg_key, section_pk)
+            let (session_id, session) = ref_mut_multi.pair_mut();
+            session.handle_timeout(node, session_id, section_pk)
         } else {
             Ok(vec![])
         }
@@ -167,26 +174,37 @@ impl DkgVoter {
     pub(crate) async fn process_message(
         &self,
         node: &Node,
-        dkg_key: &DkgSessionId,
+        session_id: &DkgSessionId,
         message: DkgMessage,
         section_pk: BlsPublicKey,
     ) -> Result<Vec<Command>> {
-        if let Some(mut session) = self.sessions.get_mut(dkg_key) {
-            session.process_message(node, dkg_key, message, section_pk)
+        let mut commands = Vec::new();
+
+        if let Some(mut session) = self.sessions.get_mut(session_id) {
+            // There is chance that when there is too many messages in the backlog,
+            // during the handling of them there will be new message reached,
+            // which be pushed to the backlog.
+            trace!("Draining backlog before process message");
+            for message in self.backlog.write().await.take(session_id).into_iter() {
+                commands.extend(session.process_message(node, session_id, message, section_pk)?);
+            }
+
+            commands.extend(session.process_message(node, session_id, message, section_pk)?)
         } else {
-            self.backlog.write().await.push(*dkg_key, message);
-            Ok(vec![])
+            trace!("Pushing to backlog {:?} - {:?}", session_id, message);
+            self.backlog.write().await.push(*session_id, message);
         }
+        Ok(commands)
     }
 
     pub(crate) fn process_failure(
         &self,
-        dkg_key: &DkgSessionId,
+        session_id: &DkgSessionId,
         failed_participants: &BTreeSet<XorName>,
         signed: DkgFailureSig,
     ) -> Option<Command> {
         self.sessions
-            .get_mut(dkg_key)?
-            .process_failure(dkg_key, failed_participants, signed)
+            .get_mut(session_id)?
+            .process_failure(session_id, failed_participants, signed)
     }
 }
