@@ -8,7 +8,6 @@
 
 use super::Prefix;
 use crate::messaging::data::Error as ErrorMessage;
-pub use crate::routing::dkg::proposal::ProposalError;
 use crate::types::{convert_dt_error_to_error_message, DataAddress, PublicKey};
 use secured_linked_list::error::Error as SecuredLinkedListError;
 use std::io;
@@ -23,23 +22,22 @@ pub type Result<T, E = Error> = std::result::Result<T, E>;
 #[derive(Debug, Error)]
 #[allow(missing_docs)]
 pub enum Error {
+    #[error("There was a problem during acquisition of a tokio::sync::semaphore permit.")]
+    PermitAcquisitionFailed,
     #[error("Failed signature check.")]
     FailedSignature,
     #[error("Section authority provider cannot be trusted: {0}")]
     UntrustedSectionAuthProvider(String),
     #[error("Proof chain cannot be trusted: {0}")]
     UntrustedProofChain(String),
+    #[error("Invalid genesis key of provided prefix map: {}", hex::encode(_0.to_bytes()))]
+    InvalidGenesisKey(bls::PublicKey),
     #[error("Cannot route. Delivery group size: {}, candidates: {}.", _0, _1)]
     CannotRoute(usize, usize),
     #[error("Empty recipient list")]
     EmptyRecipientList,
     #[error("Could not connect to any bootstrap contact")]
     BootstrapFailed,
-    // #[error("The config is invalid: {err}")]
-    // InvalidQp2pConfig {
-    //     #[source]
-    //     err: InternalConfigError,
-    // },
     #[error("Cannot connect to the endpoint: {err}")]
     CannotConnectEndpoint {
         #[from]
@@ -60,8 +58,8 @@ pub enum Error {
     InvalidMessage,
     #[error("A signature share is invalid.")]
     InvalidSignatureShare,
-    #[error("The secret key share is missing.")]
-    MissingSecretKeyShare,
+    #[error("The secret key share is missing for public key {0:?}")]
+    MissingSecretKeyShare(bls::PublicKey),
     #[error("Failed to send a message to {0}, {1}")]
     FailedSend(SocketAddr, XorName),
     #[error("Connection closed locally")]
@@ -70,8 +68,6 @@ pub enum Error {
     InvalidSectionChain(#[from] SecuredLinkedListError),
     #[error("Messaging protocol error: {0}")]
     Messaging(#[from] crate::messaging::Error),
-    #[error("proposal error: {0}")]
-    ProposalError(#[from] ProposalError),
     #[error("invalid payload")]
     InvalidPayload,
     #[error("Routing is set to not allow taking any new node")]
@@ -133,12 +129,6 @@ pub enum Error {
     /// No mapping to sn_messages::Error could be found. Either we need a new error there, or we need to handle or convert this error before sending it as a message
     #[error("No mapping to sn_messages error is set up for this NodeError {0}")]
     NoErrorMapping(String),
-    /// Logic error.
-    #[error("Logic error: {0}")]
-    Logic(String),
-    /// Timeout when trying to join the network
-    #[error("Timeout when trying to join the network")]
-    JoinTimeout,
     /// Configuration error.
     #[error("Configuration error: {0}")]
     Configuration(String),
