@@ -7,10 +7,14 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use crate::messaging::{
-    system::{DkgFailureSigSet, KeyedSig, Proposal, Section, SystemMsg},
-    DstLocation, MessageId, NodeMsgAuthority, SectionAuthorityProvider, WireMsg,
+    system::{DkgFailureSigSet, KeyedSig, Proposal, SystemMsg},
+    DstLocation, MessageId, NodeMsgAuthority, WireMsg,
+};
+use crate::routing::{
+    network_knowledge::{NetworkKnowledge, SectionAuthorityProvider, SectionKeyShare},
+    node::Node,
+    Peer, XorName,
 };
-use crate::routing::{node::Node, routing_api::Peer, section::SectionKeyShare, XorName};
 use bls::PublicKey as BlsPublicKey;
 use bytes::Bytes;
 use custom_debug::Debug;
@@ -23,12 +27,14 @@ use std::{
 
 /// Command for node.
 #[allow(clippy::large_enum_variant)]
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub(crate) enum Command {
     /// Handle `message` from `sender`.
     /// Holding the WireMsg that has been received from the network,
     HandleMessage {
-        sender: SocketAddr,
+        // This is the only command that uses `SocketAddr` instead of `Peer`, because we haven't
+        // deserialized/verified the src at this stage.
+        sender_addr: SocketAddr,
         wire_msg: WireMsg,
         #[debug(skip)]
         // original bytes to avoid reserializing for entropy checks
@@ -37,7 +43,7 @@ pub(crate) enum Command {
     // TODO: rename this as/when this is all node for clarity
     /// Handle Node, either directly or notify via event listener
     HandleSystemMessage {
-        sender: SocketAddr,
+        sender: Peer,
         msg_id: MessageId,
         msg: SystemMsg,
         msg_authority: NodeMsgAuthority,
@@ -47,29 +53,14 @@ pub(crate) enum Command {
         #[debug(skip)]
         known_keys: Vec<BlsPublicKey>,
     },
-    /// Handle verified node message after aggregation either directly or notify via event listener
-    HandleBlockingMessage {
-        sender: SocketAddr,
-        msg_id: MessageId,
-        msg: SystemMsg,
-        msg_authority: NodeMsgAuthority,
-    },
-    /// Handle Node data messages directly
-    HandleNonBlockingMessage {
-        msg_id: MessageId,
-        msg: SystemMsg,
-        msg_authority: NodeMsgAuthority,
-        dst_location: DstLocation,
-        sender: SocketAddr,
-        #[debug(skip)]
-        known_keys: Vec<BlsPublicKey>,
-    },
     /// Handle a timeout previously scheduled with `ScheduleTimeout`.
     HandleTimeout(u64),
     /// Handle peer that's been detected as lost.
-    HandlePeerLost(SocketAddr),
+    HandlePeerLost(Peer),
     /// Handle agreement on a proposal.
     HandleAgreement { proposal: Proposal, sig: KeyedSig },
+    /// Handle agree on elders. This blocks node message processing until complete.
+    HandleElderAgreement { proposal: Proposal, sig: KeyedSig },
     /// Handle the outcome of a DKG session where we are one of the participants (that is, one of
     /// the proposed new elders).
     HandleDkgOutcome {
@@ -80,7 +71,7 @@ pub(crate) enum Command {
     HandleDkgFailure(DkgFailureSigSet),
     /// Send a message to the given `recipients`.
     SendMessage {
-        recipients: Vec<(XorName, SocketAddr)>,
+        recipients: Vec<Peer>,
         wire_msg: WireMsg,
     },
     /// Parses WireMsg to send to the correct location
@@ -89,7 +80,7 @@ pub(crate) enum Command {
     PrepareNodeMsgToSend { msg: SystemMsg, dst: DstLocation },
     /// Send a message to `delivery_group_size` peers out of the given `recipients`.
     SendMessageDeliveryGroup {
-        recipients: Vec<(XorName, SocketAddr)>,
+        recipients: Vec<Peer>,
         delivery_group_size: usize,
         wire_msg: WireMsg,
     },
@@ -101,7 +92,7 @@ pub(crate) enum Command {
         /// New Node state and information
         node: Node,
         /// New section where we relocated
-        section: Section,
+        section: NetworkKnowledge,
     },
     /// Attempt to set JoinsAllowed flag.
     SetJoinsAllowed(bool),
@@ -125,7 +116,7 @@ pub(crate) enum Command {
 impl fmt::Display for Command {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            Command::HandleTimeout(_) => write!(f, "HandlePeerLost"),
+            Command::HandleTimeout(_) => write!(f, "HandleTimeout"),
             Command::ScheduleTimeout { .. } => write!(f, "ScheduleTimeout"),
             Command::HandleSystemMessage { msg_id, .. } => {
                 write!(f, "HandleSystemMessage {:?}", msg_id)
@@ -133,14 +124,9 @@ impl fmt::Display for Command {
             Command::HandleMessage { wire_msg, .. } => {
                 write!(f, "HandleMessage {:?}", wire_msg.msg_id())
             }
-            Command::HandleBlockingMessage { msg_id, .. } => {
-                write!(f, "HandleBlockingMessage {:?}", msg_id)
-            }
-            Command::HandleNonBlockingMessage { msg_id, .. } => {
-                write!(f, "HandleNonBlockingMessage {:?}", msg_id)
-            }
-            Command::HandlePeerLost(_) => write!(f, "HandlePeerLost"),
+            Command::HandlePeerLost(peer) => write!(f, "HandlePeerLost({:?})", peer.name()),
             Command::HandleAgreement { .. } => write!(f, "HandleAgreement"),
+            Command::HandleElderAgreement { .. } => write!(f, "HandleElderAgreement"),
             Command::HandleDkgOutcome { .. } => write!(f, "HandleDkgOutcome"),
             Command::HandleDkgFailure(_) => write!(f, "HandleDkgFailure"),
             Command::SendMessage { wire_msg, .. } => {
